package net.sabi.pester;

import danger.app.Application;
import danger.app.Event;
import danger.ui.AlertWindow;
import danger.ui.ActiveListView;
import danger.ui.Rect;
import danger.ui.ToolTipWindow;
import danger.util.ActiveList;
import danger.util.DEBUG;

public class AlarmListView extends ActiveListView
    implements Resources, Commands {

    // XXX some of these shouldn't conceptually be static
    private static AlarmListWindow sAlarmListWindow;
    private static AlarmSetDialog sAlarmSetDialog;

    private danger.app.Alarm mToolTipAlarm;
    private ToolTipWindow mToolTipWindow;

    public void onDecoded() {
	sAlarmListWindow = (AlarmListWindow)getWindow();
	setAutoResize(true);
	setHasBorder(false); // this doesn't work in the rsrc file
	setList(Alarms.getList());
	if (alarmsSet() > 0)
	    setHasBorder(true);
	mToolTipAlarm = new danger.app.Alarm(0, this);
	super.onDecoded();
    }

    protected AlarmSetDialog alarmSetDialog() {
	if (sAlarmSetDialog == null)
	    sAlarmSetDialog = AlarmSetDialog.getDialog();
	return sAlarmSetDialog;
    }

    public boolean eventWidgetUp(int widget, Event e) {
	switch (widget) {
	case Event.DEVICE_BUTTON_BACK:
	case Event.DEVICE_BUTTON_CANCEL:
	    Application.getCurrentApp().returnToLauncher();
	    return true;
	}
	return super.eventWidgetUp(widget, e);
    }

    public boolean receiveEvent(Event e) {
	switch (e.type) {
	case EVENT_NEW_ALARM:
	    alarmSetDialog().editAlarm(new Alarm(), true);
	    return true;
	case EVENT_CONFIRM_DISCARD_ALARM:
	    AlertWindow alert =
		Application.getCurrentApp().getAlert(ID_DISCARD_ALERT, this);
	    // XXX localize
	    alert.setMessage("Permanently discard \u201c" +
			     ((Alarm)getFocusedItem()).getMessage() + 
			     "\u201d?");
	    alert.show();
	    return true;
	case EVENT_DISCARD_ALARM:
	    Alarms.removeAlarm((Alarm)getFocusedItem());
	    return true;
	case Event.EVENT_ALARM: // tooltip invalid
	    // XXX should do this too if the time changes
	    showToolTip();
	    return true;
	}
	return super.receiveEvent(e);
    }

    protected void itemActivated(Object item) {
	if (item == null)
	    return;
	alarmSetDialog().editAlarm((Alarm)item, false);
    }

    protected void itemFocused(Object item) {
	mToolTipAlarm.deactivate();
    }

    public void loseFocus() {
	mToolTipAlarm.deactivate();
	super.loseFocus();
    }

    public void showToolTip() {
	Alarm alarm = (Alarm)getFocusedItem();
	if (alarm == null)
	    return;
	Rect rect = new Rect();
	rect = localToGlobal(getInterestingRect(rect));
	if (mToolTipWindow != null)
	    mToolTipWindow.hide();
	mToolTipWindow =
	    ToolTipWindow.showToolTip(alarm.getDateTimeString() + " - " +
				      alarm.getIntervalString(),
				      10, rect.bottom, 1, sAlarmListWindow);
	int secondsUntilUpdate = alarm.getSecondsUntilNextIntervalStringUpdate();
	if (secondsUntilUpdate == 0)
	    return;
	mToolTipAlarm.resetWake(secondsUntilUpdate);
    }

    public int alarmsSet() {
	return getListSize();
    }
    public void onItemAdded(ActiveList list, Object item, int index) {
	sAlarmListWindow.updateAlarmCount();
	super.onItemAdded(list, item, index);
	// super's default behavior is to preserve the existing selection,
	// which is admirable in general, but not what we want
	setFocus(index);
	setHasBorder(true);
    }
    public void onItemRemoved(ActiveList list, Object item, int index) {
	((Alarm)item).cancel();
	sAlarmListWindow.updateAlarmCount();
	super.onItemRemoved(list, item, index);
	if (alarmsSet() == 0)
	    setHasBorder(false);
    }
}
